#!/usr/bin/env python
#Copyright 2009 Diego Duclos
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''
This script cherrypicks the icons needed by pyfa from a full icon package
Full icon packages can be found at http://wiki.eve-id.net/CCP_Database_Dump_Resources#Images_.26_Icons

@author: cncfanatics
'''
from model import category, group, item, marketGroup, graphic, attribute, sql
import tarfile
import os
import os.path

sql.connect("sqlite")
#This script figures out all icons needed by the app and builds a tar.gz for em
def toIconPath(format, icon):
    return os.path.join(format.replace("x", "_"), "icon" + icon + ".png")

pathToIcons = raw_input("""Path to folder with icons ? [Add a / at the end]\n
This folder has three sub folders in them, 16_16, 32_32, 64_64, 128_128 and is usualy located in a subfolder called 'icons'.
A dump this script can use can be found at http://wiki.eve-id.net/CCP_Database_Dump_Resources#Images_.26_Icons\n""")
neededIcons = {}
#Figure out the icons needed by the implant part of the character editor
print "checking skills"
skillCat = category.getCategory(name = "Skill")
neededIcons[toIconPath("16x16", skillCat.icon)] = True

print "checking ships"
shipCat = category.getCategory(name = "Ship")
shipGroups = group.getGroupsByCategory(shipCat.ID)
for grp in shipGroups:
    if grp.published == 1:
        items = item.getItemsByGroup(group = grp)
        for i in items:
            for attr in i.attributes.itervalues():
                if isinstance(attr, attribute.attribute):
                    if attr.icon: neededIcons[toIconPath("16x16", attr.icon)] = True

#Figure what icons are needed by the item browser in the main window
def getGroupChildrenIcons(mGroup, buildExtraGroups = False):
    if mGroup != None:
        if mGroup.graphicID != None:
            icon = graphic.getIconName(mGroup.graphicID)
            neededIcons[toIconPath("16x16", icon)] = True
            
        if mGroup.hasTypes == 1:
            items = item.getItemsByGroup(marketGroup = mGroup)
            for i in items:
                if buildExtraGroups and i.group:
                    neededIcons[toIconPath("16x16", i.group.icon)] = True
                if i.attributes:
                    for attr in i.attributes.itervalues():
                        if isinstance(attr, attribute.attribute):
                            if attr.icon: neededIcons[toIconPath("16x16", attr.icon)] = True
                            
                if i.icon != None:
                    neededIcons[toIconPath("16x16", i.icon)] = True 
        else:
            if mGroup.children != None:
                for child in mGroup.children.itervalues():
                    getGroupChildrenIcons(child, buildExtraGroups)

print "checking modules"
mg = marketGroup.getMarketGroup(name = "Ship Equipment")
getGroupChildrenIcons(mg)
print "checking drones"
mg = marketGroup.getMarketGroup(name = "Drones")
getGroupChildrenIcons(mg)
print "checking rigs"
mg = marketGroup.getMarketGroup(name = "Rigs")
getGroupChildrenIcons(mg)
print "checking ammo"
mg = marketGroup.getMarketGroup(name = "Ammunition & Charges")
getGroupChildrenIcons(mg, buildExtraGroups = True)
#Can't look by name as there's 2 marketgroups named subsystems
print "checking subsystems"
mg = marketGroup.getMarketGroup(ID = 1112)
getGroupChildrenIcons(mg)
print "checking implants & boosters"
mg = marketGroup.getMarketGroup(name = "Implants & Boosters")
getGroupChildrenIcons(mg)

#We got all the icons, let's build an archive with gzip
if os.path.exists("icons.tar.bz2"):
    os.remove("icons.tar.bz2")
print "making tarfile"
tf = tarfile.open("icons.tar.bz2", "w:bz2")
for icon in neededIcons.iterkeys():
    icon = icon.replace(" ", "")
    try:
        tf.add(pathToIcons + icon, os.path.join("icons", icon))
    except Exception:
        print("couldn't add " + pathToIcons + icon)
    
tf.close()
print "done"
